<?php
/**
 * Adaptive Addon Batch Update
 *
 * @package     Adaptive Addon
 * @since       2.1.3
 */

if ( ! class_exists( 'Adaptive_Addon_Background_Updater' ) ) {

	/**
	 * Adaptive_Addon_Background_Updater Class.
	 */
	class Adaptive_Addon_Background_Updater {

		/**
		 * Background update class.
		 *
		 * @var object
		 */
		private static $background_updater;

		/**
		 * DB updates and callbacks that need to be run per version.
		 *
		 * @var array
		 */
		private static $db_updates = array(
			'2.2.0' => array(
				'adaptive_addon_page_builder_button_color_compatibility',
			),
			'2.3.0' => array(
				'adaptive_addon_page_header_parallax_device',
			),
			'2.3.3' => array(
				'adaptive_addon_css_gen_multi_site_fix',
			),
			'2.4.0' => array(
				'adaptive_responsive_content_background_option',
				'adaptive_addon_update_theme_tablet_breakpoint',
			),
			'2.5.0' => array(
				'custom_layout_compatibility_having_code_posts',
				'adaptive_addon_page_header_submenu_color_options',
			),
			'2.6.0' => array(
				'adaptive_addon_header_css_optimizations',
			),
			'3.5.0' => array(
				'adaptive_addon_page_headers_support_to_builder_layout',
			),
			'3.5.1' => array(
				'adaptive_addon_headings_font_support',
				'adaptive_addon_cart_color_not_working_in_old_header',
			),
			'3.5.7' => array(
				'adaptive_addon_outline_cart_bg_color_support',
				'adaptive_addon_remove_header_sections_deps_new_builder',
				'adaptive_addon_swap_section_not_working_in_old_header',
			),
			'3.5.8' => array(
				'adaptive_sticky_header_site_title_tagline_css',
			),
			'3.5.9' => array(
				'adaptive_addon_remove_responsive_account_menu_colors_support',
			),
		);

		/**
		 *  Constructor
		 */
		public function __construct() {

			// Addon Updates.
			if ( is_admin() ) {
				add_action( 'admin_init', array( $this, 'install_actions' ) );
			} else {
				add_action( 'gc', array( $this, 'install_actions' ) );
			}

			// Core Helpers - Batch Processing.
			require_once ADAPTIVE_EXT_DIR . 'classes/library/batch-processing/gc-async-request.php';
			require_once ADAPTIVE_EXT_DIR . 'classes/library/batch-processing/gc-background-process.php';
			require_once ADAPTIVE_EXT_DIR . 'classes/library/batch-processing/class-gc-background-process-adaptive-addon.php';

			self::$background_updater = new GC_Background_Process_Adaptive_Addon();
		}

		/**
		 * Check if database is migrated
		 *
		 * @since 2.3.2
		 *
		 * @return true If the database migration should not be run through CRON.
		 */
		public function check_if_data_migrated() {

			$fallback = false;

			$is_db_version_updated = $this->is_db_version_updated();
			if ( ! $is_db_version_updated ) {

				$db_migrated = get_transient( 'adaptive-addon-db-migrated' );

				if ( ! $db_migrated ) {
					$db_migrated = array();
				}

				array_push( $db_migrated, $is_db_version_updated );
				set_transient( 'adaptive-addon-db-migrated', $db_migrated, 3600 );

				$db_migrate_count = count( $db_migrated );
				if ( $db_migrate_count >= 5 ) {

					$customizer_options = get_option( 'adaptive-settings' );

					// Get all customizer options.
					$version_array = array(
						'is_adaptive_addon_queue_running' => false,
					);

					// Merge customizer options with version.
					$adaptive_options = gc_parse_args( $version_array, $customizer_options );

					update_option( 'adaptive-settings', $adaptive_options );

					$fallback = true;
				}
			}
			return $fallback;
		}

		/**
		 * Checks if adaptive addon version is updated in the database
		 *
		 * @since 2.3.2
		 *
		 * @return true if adaptive addon version is updated.
		 */
		public function is_db_version_updated() {
			// Get auto saved version number.
			$saved_version = Adaptive_Addon_Update::adaptive_addon_stored_version();

			return version_compare( $saved_version, ADAPTIVE_EXT_VER, '=' );
		}

		/**
		 * Check Cron Status
		 *
		 * Gets the current cron status by performing a test spawn. Cached for one hour when all is well.
		 *
		 * @since 2.3.0
		 *
		 * @return true if there is a problem spawning a call to Gc-Cron system.
		 */
		public function test_cron() {

			global $gc_version;

			if ( defined( 'DISABLE_GC_CRON' ) && DISABLE_GC_CRON ) {
				return true;
			}

			if ( defined( 'ALTERNATE_GC_CRON' ) && ALTERNATE_GC_CRON ) {
				return true;
			}

			$cached_status = get_transient( 'adaptive-addon-cron-test-ok' );

			if ( $cached_status ) {
				return false;
			}

			$sslverify     = version_compare( $gc_version, 4.0, '<' );
			$doing_gc_cron = sprintf( '%.22F', microtime( true ) );

			$cron_request = apply_filters(
				'cron_request', // phpcs:ignore GeChiUI.NamingConventions.PrefixAllGlobals.NonPrefixedHooknameFound
				array(
					'url'  => site_url( 'gc-cron.php?doing_gc_cron=' . $doing_gc_cron ),
					'args' => array(
						'timeout'   => 3,
						'blocking'  => true,
						'sslverify' => apply_filters( 'https_local_ssl_verify', $sslverify ), // phpcs:ignore GeChiUI.NamingConventions.PrefixAllGlobals.NonPrefixedHooknameFound
					),
				)
			);

			$result = gc_remote_post( $cron_request['url'], $cron_request['args'] );

			if ( gc_remote_retrieve_response_code( $result ) >= 300 ) {
				return true;
			} else {
				set_transient( 'adaptive-addon-cron-test-ok', 1, 3600 );
				return false;
			}

			return $migration_fallback;
		}

		/**
		 * Install actions when a update button is clicked within the admin area.
		 *
		 * This function is hooked into admin_init to affect admin and gc to affect the frontend.
		 *
		 * @since 2.1.3
		 * @return void
		 */
		public function install_actions() {

			if ( true === $this->is_new_install() ) {
				self::update_db_version();
				return;
			}

			$customizer_options = get_option( 'adaptive-settings' );

			$fallback         = $this->test_cron();
			$db_migrated      = $this->check_if_data_migrated();
			$is_queue_running = ( isset( $customizer_options['is_adaptive_addon_queue_running'] ) && '' !== $customizer_options['is_adaptive_addon_queue_running'] ) ? $customizer_options['is_adaptive_addon_queue_running'] : false;

			$fallback = ( $db_migrated ) ? $db_migrated : $fallback;

			if ( $this->needs_db_update() && ! $is_queue_running ) {
				$this->update( $fallback );
			} else {
				if ( ! $is_queue_running ) {
					self::update_db_version();
				}
			}

		}

		/**
		 * Is this a brand new addon install?
		 *
		 * @since 2.1.3
		 * @return boolean
		 */
		private function is_new_install() {

			// Get auto saved version number.
			$saved_version = Adaptive_Addon_Update::adaptive_addon_stored_version();

			if ( false === $saved_version ) {
				return true;
			} else {
				return false;
			}
		}

		/**
		 * Is a DB update needed?
		 *
		 * @since 2.1.3
		 * @return boolean
		 */
		private function needs_db_update() {

			$updates = $this->get_db_update_callbacks();

			if ( empty( $updates ) ) {
				return false;
			}

			$customizer_options = get_option( 'adaptive-settings' );

			$addon_auto_version = ( isset( $customizer_options['adaptive-addon-auto-version'] ) && '' !== $customizer_options['adaptive-addon-auto-version'] ) ? $customizer_options['adaptive-addon-auto-version'] : null;

			return ! is_null( $addon_auto_version ) && version_compare( $addon_auto_version, max( array_keys( $updates ) ), '<' );
		}

		/**
		 * Get list of DB update callbacks.
		 *
		 * @since 2.1.3
		 * @return array
		 */
		public function get_db_update_callbacks() {
			return self::$db_updates;
		}

		/**
		 * Push all needed DB updates to the queue for processing.
		 *
		 * @param bool $fallback Fallback migration.
		 */
		private function update( $fallback ) {

			$current_db_version = Adaptive_Addon_Update::adaptive_addon_stored_version();

			error_log( 'Adaptive Addon: Batch Process Started!' ); // phpcs:ignore GeChiUI.PHP.DevelopmentFunctions.error_log_error_log
			if ( count( $this->get_db_update_callbacks() ) > 0 ) {
				foreach ( $this->get_db_update_callbacks() as $version => $update_callbacks ) {
					if ( version_compare( $current_db_version, $version, '<' ) ) {
						foreach ( $update_callbacks as $update_callback ) {
							if ( $fallback ) {
								call_user_func( $update_callback );
							} else {
								error_log( sprintf( 'Adaptive Addon: Queuing %s - %s', $version, $update_callback ) ); // phpcs:ignore GeChiUI.PHP.DevelopmentFunctions.error_log_error_log
								self::$background_updater->push_to_queue( $update_callback );
							}
						}
					}
				}

				if ( $fallback ) {
					error_log( 'Adaptive Addon: Running migration without batch processing.' ); // phpcs:ignore GeChiUI.PHP.DevelopmentFunctions.error_log_error_log
					self::update_db_version();
				} else {
					$customizer_options = get_option( 'adaptive-settings' );

					// Get all customizer options.
					$version_array = array(
						'is_adaptive_addon_queue_running' => true,
					);

					// Merge customizer options with version.
					$adaptive_options = gc_parse_args( $version_array, $customizer_options );

					update_option( 'adaptive-settings', $adaptive_options );

					self::$background_updater->push_to_queue( 'update_db_version' );
				}
			} else {
				self::$background_updater->push_to_queue( 'update_db_version' );
			}
			self::$background_updater->save()->dispatch();
		}

		/**
		 * Update DB version to current.
		 *
		 * @param string|null $version New Adaptive addon version or null.
		 */
		public static function update_db_version( $version = null ) {

			do_action( 'adaptive_addon_update_before' );

			// Get auto saved version number.
			$saved_version       = Adaptive_Addon_Update::adaptive_addon_stored_version();
			$adaptive_addon_version = ADAPTIVE_EXT_VER;

			if ( false === $saved_version ) {

				// Get all customizer options.
				$customizer_options = get_option( 'adaptive-settings' );

				// Get all customizer options.
				/* Add Current version constant "ADAPTIVE_EXT_VER" here after 1.0.0-rc.9 update */
				$version_array = array(
					'adaptive-addon-auto-version' => ADAPTIVE_EXT_VER,
				);
				$saved_version = ADAPTIVE_EXT_VER;

				// Merge customizer options with version.
				$adaptive_options = gc_parse_args( $version_array, $customizer_options );

				// Update auto saved version number.
				update_option( 'adaptive-settings', $adaptive_options );

			}

			// If equals then return.
			if ( version_compare( $saved_version, ADAPTIVE_EXT_VER, '=' ) ) {

				// Get all customizer options.
				$customizer_options = get_option( 'adaptive-settings' );

				// Get all customizer options.
				$options_array = array(
					'is_adaptive_addon_queue_running' => false,
				);

				// Merge customizer options with version.
				$adaptive_options = gc_parse_args( $options_array, $customizer_options );

				// Update auto saved version number.
				update_option( 'adaptive-settings', $adaptive_options );

				return;
			}

			$adaptive_addon_version = ADAPTIVE_EXT_VER;

			// Get all customizer options.
			$customizer_options = get_option( 'adaptive-settings' );

			// Get all customizer options.
			$options_array = array(
				'adaptive-addon-auto-version'     => $adaptive_addon_version,
				'is_adaptive_addon_queue_running' => false,
			);

			// Merge customizer options with version.
			$adaptive_options = gc_parse_args( $options_array, $customizer_options );

			// Update auto saved version number.
			update_option( 'adaptive-settings', $adaptive_options );

			error_log( 'Adaptive Addon: DB version updated!' ); // phpcs:ignore GeChiUI.PHP.DevelopmentFunctions.error_log_error_log

			// Update variables.
			Adaptive_Theme_Options::refresh();

			// Refresh Adaptive Addon CSS and JS Files on update.
			Adaptive_Minify::refresh_assets();

			delete_transient( 'adaptive-addon-db-migrated' );

			do_action( 'adaptive_addon_update_after' );
		}
	}
}


/**
 * Kicking this off by creating a new instance
 */
new Adaptive_Addon_Background_Updater();
